/*
 * React JQuery plugin
 * https://github.com/natedavisolds/react.js
 *
 * Copyright 2011, Nathan Davis Olds
 * Distributed under the MIT license.
*/

(function($){

  $.fn.reactTo = function(selector) {
    var $elements = $(selector),
        $reactor_element = $(this)

    var _proxy_event = function() {
      $reactor_element.trigger('react.reactor');
      return true
    };

    $elements.filter(':not(:text), :not(:password)').on('change.reactor', _proxy_event);
    $elements.filter(':text.date-picker').on('change.reactor', _proxy_event);
    $elements.filter(':text').on('keyup.reactor', _proxy_event);
    $elements.filter(':password').on('keyup.reactor', _proxy_event);

    return this;
  };

  $.fn.reactIf = function(sel, exp_func) {
    var $sel = $(sel);
    var args = Array.prototype.slice.call( arguments, 2 );

    var _func = function() {
      if ($.isFunction(exp_func)) {
        return exp_func.apply($sel);
      } else {
        var _returned = $.fn.reactor.helpers[exp_func].apply($sel, args);

        if ($.isFunction(_returned)) {
          return _returned.apply($sel)
        } else {
          return _returned;
        }
      }
    };

    this.each(function() {
      var $reactor = $(this);

      if (!$reactor.hasClass('reactor')) { $reactor.reactor(); }

      var conditions_arry = $reactor.data('conditions.reactor');
      if (!$.isArray(conditions_arry)) { conditions_arry = [[]]; }

      conditions_arry[0].push(_func);

      $reactor.data('conditions.reactor', conditions_arry);
    });

    $(this).reactTo(sel);

    return this;
  };

  $.fn.orReactIf = function(sel, exp_func, arg1, arg2) {
    this.each(function() {
      var $element = $(this),
          conditions = $element.data('conditions.reactor');

      conditions.unshift([])
      $element.data('conditions.reactor', conditions);
    })      

    if (sel && exp_func) {
      this.reactIf(sel, exp_func, arg1, arg2)      
    }

    return this;
  }

  $.fn.react = function() {
    this.each(function() {
      $(this).trigger('react.reactor');
    });

    return this;
  };

  $.fn.reactor = function(options) {
    var settings = $.extend({}, $.fn.reactor.defaults, options);

    this.each(function() {
      var $element = $(this);

      if (!$.isArray($element.data('conditions.reactor'))) { 
        $element
          .data('conditions.reactor', [[]])
          .addClass('reactor'); 
      }

      var allAreTrue = function(candidate) {
        var result = true 

        $.each(candidate, function(index, func_condition) {
          if (!(func_condition.call())) { 
            result = false
          }
        })

        return result
      }

      var isReactionary = function() {
        var conditionalArray = $(this).data('conditions.reactor');
        var result = false;

        $.each(conditionalArray, function(index, logicalOrConditions) {
          result = result || allAreTrue(logicalOrConditions)
        })

        return result
      };

      var reaction = function(evt) {
        if (isReactionary.apply(this)) {
          settings.compliant.apply($element);
        } else {
          settings.uncompliant.apply($element);
        }
      };

      $element
        .off('react.reactor')
        .on('react.reactor', reaction);
    });

    return this;
  };

  $.fn.reactor.defaults = {
    compliant: function() {
      $(this).show();
    },
    uncompliant: function() {
      $(this).hide();
    }
  };

  $.fn.reactor.helpers = {
    NotBlank: function() {
      return( this.val().toString() !== "" );
    },

    Blank: function() {
      return( this.val().toString() === "" );
    },

    Disabled: function() {
      return( this.filter(':disabled').length > 0 );
    },

    Enabled: function() {
      return( this.filter(':enabled').length > 0 );
    },

    EqualTo: function(matchStr) {
      var _func = function() {
        var v = this.val();
        if (v) { return( v.toString() == matchStr ); }
        else { return false; }
      };

      return _func;
    },

    LessThan: function(number) {
      var _func = function() {
        var v = this.filter('span').length > 0 ? this.text() : this.val();
        return( parseInt(v) < number );
      }

      return _func;
    },

    MoreThan: function(number) {
      var _func = function() {
        var v = this.filter('span').length > 0 ? this.text() : this.val();
        return( parseInt(v) > number );
      }

      return _func;
    },

    Between: function(min, max) {
        var _func = function() {
            var v = this.val();
            return(!(v && (parseInt(v, 10) > max || parseInt(v, 10) < min)));
        };

        return _func;
    },

    BetweenSameLength: function(min, max) {
        var len = min.toString().length,
            _func = function() {
              var v = this.val();
              return(!(v && v.length == len && (parseInt(v, 10) > max || parseInt(v, 10) < min)));
            };

        return _func;
    },

    IsChecked: function() {
      return this.is(':checked');
    },

    IsNotChecked: function() {
      return !this.is(':checked');
    },

    NumberOfDigitsIs: function(number) {
      var comparisonString = this.val().toString().replace(/[^\d]+/g,''),
          passing = false,
          length = comparisonString.length

      for(index in arguments) {
        if (length == arguments[index]) {
          passing = true
        }
      }

      return passing
    },

    HasElements: function() {
      return this.size() > 0
    },

    HasValueWhenVisible: function() {
      if (this.is(':visible')) {
        return( this.val().toString() != "" && parseFloat(this.val()) != 0.0)
      } else {
        return true
      }
    }
  };
})(jQuery);